% Reduction when A is not full rank. K and R need not be full rank. Input 
% K (state cost matrix), KT (final period "scrap value" state cost matrix)
% A (transition matrix), C (control multiplier matrix), and R (control cost matrix). 
% Note: variable names are case sensitive.
% K:    n by n
% KT:   n by n
% A:    n by n
% C:    n by k
% R:    k by k
%function outputs are the reduced-order A,C,K,R matrices plus a
%transformation matrix to correct the results for the reduced problem.
%Format is: [Aa, Ca, Ka, Ra, Ta]=RedA(A,C,K,R,c)
%To start read in these 5 matrices (you may set KT=c*K and RT=c*R)
%and then type [Aa,Ka,Ca,Ra,Ta]=RedA(A,C,K,R,c)

function [Ad,Cd,Kd,Rd,Transfo1,Transfo2]=RedA(A,C,K,R,c)

% First determine n and k for future use
[n,k]=size(C);

% Check if K and R are symmetric
% Check if C has full column rank and if necessary second-order conditions are met 

if rank (C'*K*C+R)==k
    %((norm(K'-K,1)/norm(K,1)) < eps & (norm(R'-R,1)/norm(R,1)) < eps & (rank(C)==k & rank (C'*K*C + R)==k))
	disp('C`KC + R is invertible '); 
else
	disp('Error: Read in matrices again');
    pause
end;

ra=rank(A)
[Pa]=SubNotInvertible(A); 
Ap =Pa*A*Pa';
A1=Ap(1:ra,1:ra);
A2=Ap(1:ra,ra+1:n);
A3=Ap(ra+1:n, 1:ra);
Ta=[eye(ra) inv(A1)*A2; zeros(n-ra, ra) eye(n-ra)];
Tb=[A1 zeros(ra, n-ra); A3 eye(n-ra)];
Aa=Ta*Ap*inv(Ta);
Ca=Ta*Pa*C;
Ka=inv(Ta)'*Pa*K*Pa'*inv(Ta);
Ra=R;

Kb=Aa'*Ka*Aa;
Jb=Aa'*Ka*Ca;
Rb=Ra+Ca'*Ka*Ca;
Cb=Ca;

Kc=Kb-Jb*inv(Rb)*Jb';
Rc=Rb;
Ac=(eye(n)-Ca*inv(Rb)*Ca'*Ka)*Aa;
Cc=Cb;

Ad=[eye(ra) zeros(ra, n-ra)]*(eye(n)-Ca*inv(Rb)*Ca'*Ka)*Ta*Tb*[eye(ra) zeros(ra, n-ra)]';
Kd=[eye(ra) zeros(ra, n-ra)]*Tb'*Ta'*Ka*(eye(n)-Ca*inv(Rb)*Ca'*Ka)*Ta*Tb*[eye(ra) zeros(ra, n-ra)]';
Cd=[eye(ra) zeros(ra, n-ra)]*Cc;
Rd=Rc;

Transfo1=[eye(ra) zeros(ra, n-ra)]*Ta*Pa;

Transfo2=(inv(Rb)*Ca'*Ka*Ta*Tb*[eye(ra) zeros(ra, n-ra)]');

% f=(Transfo2 + fa)*Transfo1
% p=Transfo1'*pa*Transfo1 + K

% ending of file RedA.m
% Copyright July 2003, Ronald J. Balvers, West Virginia University.