function [Ak,Kk,Ck,Rk,TransfoK]=RedK(A,C,K,c)

% Reduction when K is not full rank. A need not be full rank. Initial R=0. Input: 
% K (state cost matrix), KT (final period "scrap value" state cost matrix)
% A (transition matrix), C (control multiplier matrix).
% Note: variable names are case sensitive.
% K:    n by n
% KT:   n by n
% A:    n by n
% C:    n by k
%function outputs are reduced-order A,C,K,R matrices plus a
%transformation matrix to correct the results for the reduced problem.
%Format is: [Ak, Ck, Kk, Rk, TransfoK]=RedK(A,C,K,c)
%To start read in these 4 matrices (set KT=c*K)
%and then type [Ak,Kk,Ck,Rk,TransfoK]=RedK(A,C,K,c)

% First determine n and k for future use
[n,k]=size(C);

% Check if K is symmetric
% Check if C has full column rank and if second-order conditions are met 

if ((norm(K'-K,1)/norm(K,1)) < eps & (rank(C)==k & rank (C'*K*C)==k))
	disp('C`KC is invertible '); 
else
	disp('Error: Read in matrices again');
end

M = null(C','r')
X=(eye(n)-C*inv(C'*K*C)*C'*K)*M*inv(M'*M)
Kk=inv(M'*M)*M'*K*X;
Ak=M'*A*X;
Ck=M'*A*C;
Rk=C'*K*C;
TransfoK=M'*A;
    
% ending of file RedK.m
% Copyright July 2003, Ronald J. Balvers, West Virginia University.
    

