% The problem must be in Chow form (that is, the control variables have to
% be included in the state vector).
% The user must input T (number of periods until the horizon)
% K (state cost matrix), KT (final period "scrap value" state cost matrix)
% A (transition matrix), C (control multiplier matrix). 
% Note: variable names are case sensitive.
% K:    n by n
% KT:   n by n
% A:    n by n
% C:    n by k
% T: 	1 by 1
%function outputs are the time sequences of:
%Feedback matrix Ft, Riccati matrix Ht, Kernel matrix
%Phit.  All but the first are optional.
%Format is: [F,H,Phi]=RedM(A,C,K,KT,T)
%To start read in these 4 matrices (you may set KT=K), 1 vector, and 1 scalar now
%and then type [F,H]=RedM(A,C,K,KT,T):

function [F,H,Phi]=RedM(A,C,K,KT,T)

% First determine n and k for future use
[n,k]=size(C);

% Check if K is symmetric positive definite
% Check if C has full column rank and if second-order conditions are met 

if (rank(K)==n & (norm(K'-K,1)/norm(K,1)) < eps & (rank(C)==k))
	disp('K is symmetric and invertible and C`KC is positive definite '); 
else
	disp('First run RedMain to make K invertible or Error: Read in matrices again');
    pause
end
    
q=n-k;
Iq=eye(q);
C2=C(q+1:n,:);

% Guarantee that C2 is invertible
[P]=C2NotInvertible(C); 	
    C=P*C;
	A=P*A*P';
	K=P*K*P';
	C2=C(q+1:n,:);
%note that, with these transformations, P has no impact on the maximized
%objective or on the optimal control vector over time.
C1=C(1:q,:);

M=[Iq;-inv(C2')*C1'];
B1=M'*inv(K)*M;
B2=M'*A*inv(K)*M
B3=M'*A*inv(K)*A'*M;

%Does B2 have full rank?
%Solve for Pht recursively, if B2 has full rank;
%Compute Ft using Pht
%Save all Ft and Pht in three-dimensional arrays FFF and PPP;

if Rank(B2)==q
	disp('B2 has full rank');
    InvPhiT=M'*inv(KT)*M;
    InvPhit=InvPhiT;
    Phit=inv(InvPhit);
    i=T;
	Iw=eye(n);
	W=inv(C'*C)*C'*inv(K)*(Iw-A'*M*inv(B2')*M'*inv(K));
        
      	while i>1;
          PPP(:,:,i)=Phit;   
	      InvPhit=B1-B2'*(inv(InvPhit+B3))*B2;
          Phit=inv(InvPhit);
          Ft=-W*M*Phit*M'*A+W*K*A;
          FFF(:,:,i-1)=Ft;
  	      i=i-1;
      	end;
        
     FFF(:,:,T)=inv(C'*KT*C)*C'*KT*A

    for i=2:T;
        HHH(:,:,i-1)=K+A'*M*(PPP(:,:,i))*M'*A;
    end
        HHH(:,:,T)=KT;
        
 else 
   disp('B2 is singular');
   %call function to generate permutation matrix P2 to make the upper left corner of B2 invertible
   [P2]=SubNotInvertible(B2); 
        M=[Iq;-inv(C2')*C1']*(P2');
        B1=M'*inv(K)*M;
        B2=M'*A*inv(K)*M;
        B3=M'*A*inv(K)*A'*M;
        
%partition (permuted) B2 to generate the upper left corner and the upper
%right corner to apply theorem 1*
        r=rank(B2);
        %permute B2??
        S1=B2(1:r, 1:r);
        S2=B2(1:r, r+1:q);
        Z=[eye(r);zeros(q-r,r)];
        Mstar=[eye(r);S2'*inv(S1')];
        B13i=inv(B1+B3);
        Mblk=inv(Mstar'*B13i*Mstar);
        InvPhiT=M'*inv(KT)*M;

        Bstar1=Z'*(B1-B2'*B13i*B2+B2'*B13i*Mstar*Mblk*Mstar'*B13i*B2)*Z;
        Bstar2=Mblk*Mstar'*B13i*B2*Z;
        Bstar3=Mblk-Z'*B1*Z;
        
            InvPhiStarTmin1=Z'*(B1-B2'*(inv(B3+InvPhiT))*B2)*Z;
            PPP(:,:,T)=inv(InvPhiT);
	        g=T-1;
	        InvPhiStar=InvPhiStarTmin1;

	        while g>1;
		        SSS(:,:,g)=inv(InvPhiStar);
                InvPhi=B1+Mstar*(InvPhiStar-Z'*B1*Z)*Mstar';
                PPP(:,:,g)=inv(InvPhi);
                InvPhiStar=Bstar1-Bstar2'*(inv(InvPhiStar+Bstar3))*Bstar2;
		        g=g-1;
	        end;

       while (rank(Bstar2)<r);
	      disp('Bstar2 is not invertible.  The true kernel size is calculuted and it is possible to ...')
          disp('simplify the problem further by repeated application of Theorem 1*. Here we solve using the...')
          disp('kernel of the size of Bstar2 rather than the rank of Bstar2')
       
            B1=Bstar1;
            B2=Bstar2;
            B3=Bstar3;
        
            q=r;
            r=rank(B2);
            
            [P3]=SubNotInvertible(B2); 
        
            B1=P3*B1*P3';
            disp('The final version of BStar2 is:')
            B2=P3*B2*P3'
            B3=P3*B3*P3';
                 
            S1=B2(1:r, 1:r);
            S2=B2(1:r, r+1:q);
            Z=[eye(r);zeros(q-r,r)];
            Mstar=[eye(r);S2'*inv(S1')];
            B13i=inv(B1+B3);
            Mblk=inv(Mstar'*B13i*Mstar);
        
            Bstar1=Z'*(B1-B2'*B13i*B2+B2'*B13i*Mstar*Mblk*Mstar'*B13i*B2)*Z;
            Bstar2=Mblk*Mstar'*B13i*B2*Z;
            Bstar3=Mblk-Z'*B1*Z;
        end;

        disp('Note that the minimum dynamic kernel of this problem equals:')
        r
       
        
            
     for i=2:T;
         Ht=K+A'*M*(PPP(:,:,i))*M'*A;
	     Ft=-inv(C'*Ht*C)*C'*Ht*A;
         FFF(:,:,i-1)=Ft;
         HHH(:,:,i-1)=Ht;
     end;
     FT=-inv(C'*KT*C)*C'*KT*A;
     FFF(:,:,T)=FT;
     HHH(:,:,T)=KT;
     
 end

 H=HHH
 Phi=PPP
 F=FFF