% The user must input matrices Kbar (state cost matrix), Abar (transition matrix)
% Cbar (control multiplier matrix), J (crossproduct matrix), R (control cost matrix)
% and scalars b (discount factor), c (weight on final period state cost).
% Note that all variables definitions are case sensitive.
% Kbar: n by n
% Abar: n by n
% Cbar: n by k
% J: 	n by k
% R: 	k by k
% b:    1 by 1
% Output is the size of the Riccati kernel and the full feedback and
% Riccati matrices.

[nbar,kbar]=size(Cbar);
K=[Kbar J; J' R];
A=sqrt(b)*[Abar zeros(nbar,kbar); zeros(kbar,nbar) zeros(kbar)];
C=sqrt(b)*[Cbar; eye(kbar)];
[n,k]=size(C);
display('increase in # of state variables due to adding controls as states:')
Increase=n-nbar

[Ac, Cc, Kc, S]=RedAC(A,C,K)
[nc,kc]=size(Cc);
display('Reduction in # of state variables due to noncontrollable states:')
Reduction=n-nc

[Ak,Kk,Ck,Rk,TransfoK]=RedK(Ac,Cc,Kc,c)
[nk,kk]=size(Ck);
display('Reduction in # of state variables due to R and K singularities:')
Reduction=nc-nk

Aq=Ak; Kq=Kk; Cq=Ck; Rq=Rk;
nq=nk; kq=kk;
Transfo2star=zeros(kq,nq); Transfo1star=eye(nq);
RoundA=0

while rank(Aq)<nq
    [Ad,Cd,Kd,Rd,Transfo1,Transfo2]=RedA(Aq,Cq,Kq,Rq,c)
    Aq=Ad; Cq=Cd; Kq=Kd; Rq=Rd;
    Transfo2star=Transfo2*Transfo1 + Transfo2star*Transfo1star;
    Transfo1star=Transfo1*Transfo1star;
    [nq,kq]=size(Cq);
    display('Round of reduction based on A singularities:')
    RoundA=RoundA+1
end
display('Reduction in # of state variables due to A singularities:')
Reduction=nk-nq

Aa=Aq; Ca=Cq; Ka=Kq; Ra=Rq;

[fa,pa] = olrp(b,Aa,Ca,Ka,Ra)

fk=Transfo2star + fa*Transfo1star
pk=Transfo1star'*pa*Transfo1star + Kk

fc=fk*TransfoK + inv(Cc'*Kc*Cc)*Cc'*Kc*Ac;
pc=TransfoK'*pk*TransfoK + Kc;

[f,p] = olrp(1,A,C,K,zeros(k),zeros(n,k))
[ff,pp] = olrp(b,Abar,Cbar,Kbar,R,J)

flong=f*S
plong=S'*p*S

fshort=fc
pshort=pc

% ending of file RedMainInf.m
% Copyright July 2003, Ronald J. Balvers, West Virginia University.